@@ -1,3 +1,4 @@
+import warnings
 from typing import Dict
 
 from data_spec_validator.spec.defines import BaseValidator
@@ -20,7 +21,7 @@ def register(check_2_validator_map) -> bool:
 
         if check in _custom_map:
             ori_validator = _custom_map[check]
-            print(
+            warnings.warn(
                 f'[DSV][WARNING] Check({check}) already exists, gonna overwrite the validator from '
                 f'{_get_class_name(ori_validator)} to {_get_class_name(validator)}'
             )
