@@ -1,7 +1,10 @@
+import warnings
 from abc import ABCMeta, abstractmethod
 from enum import Enum
 from functools import lru_cache, reduce
-from typing import Dict, Type, Union
+from typing import Any, Dict, List, Type, Union
+
+from .utils import raise_if
 
 # TYPE
 NONE = 'none'
@@ -25,6 +28,7 @@ DECIMAL_PLACE = 'decimal_place'
 SPEC = 'spec'
 LIST_OF = 'list_of'
 ONE_OF = 'one_of'
+FOREACH = 'foreach'
 DUMMY = 'dummy'
 ANY_KEY_EXISTS = 'any_key_exists'
 KEY_COEXISTS = 'key_coexists'
@@ -63,7 +67,6 @@ def get_default_check_2_validator_map() -> Dict[str, BaseValidator]:
     from data_spec_validator.spec.validators import (
         AmountRangeValidator,
         AmountValidator,
-        AnyKeyExistsValidator,
         BoolValidator,
         CondExistValidator,
         DateRangeValidator,
@@ -73,10 +76,10 @@ def get_default_check_2_validator_map() -> Dict[str, BaseValidator]:
         DigitStrValidator,
         DummyValidator,
         EmailValidator,
+        ForeachValidator,
         IntValidator,
         JSONBoolValidator,
         JSONValidator,
-        KeyCoexistsValidator,
         LengthValidator,
         ListOfValidator,
         ListValidator,
@@ -108,12 +111,11 @@ def get_default_check_2_validator_map() -> Dict[str, BaseValidator]:
         DATE: DateValidator(),
         DATE_RANGE: DateRangeValidator(),
         DUMMY: DummyValidator(),
-        ANY_KEY_EXISTS: AnyKeyExistsValidator(),
-        KEY_COEXISTS: KeyCoexistsValidator(),
         EMAIL: EmailValidator(),
         UUID: UUIDValidator(),
         REGEX: RegexValidator(),
         COND_EXIST: CondExistValidator(),
+        FOREACH: ForeachValidator(),
     }
 
 
@@ -134,7 +136,7 @@ def _get_check_2_validator_map() -> Dict[str, BaseValidator]:
     return validator_map
 
 
-def get_validator(check) -> Union[BaseValidator, BaseWrapper]:
+def get_validator(check: str) -> Union[BaseValidator, BaseWrapper]:
     found_idx = check.find(_wrapper_splitter)
     validator_map = _get_check_2_validator_map()
     ori_validator = validator_map.get(check[found_idx + 1 :], validator_map[DUMMY])
@@ -152,15 +154,32 @@ def get_unknown_field_value() -> UnknownFieldValue:
     return UnknownFieldValue()
 
 
+class ErrorMode(Enum):
+    MSE = 'most_significant'
+    ALL = 'all'
+
+
+class DSVError(Exception):
+    def __init__(self, *errors: List[Type[Exception]]):
+        self._errors = errors
+
+    def __str__(self, *args, **kwargs):
+        return repr(self._errors)
+
+
 class ValidateResult:
-    def __init__(self, spec=None, field=None, value=None, check=None, error=None):
+    def __init__(self, spec: Type = None, field: str = None, value: Any = None, check: str = None, error=None):
         # TODO: Output spec & check information when there's a debug message level for development.
-        self.__spec = type(spec)
+        self.__spec = spec.__name__ if spec else None
         self.__field = field
         self.__value = value
         self.__check = check
         self.__error = error
 
+    @property
+    def spec(self) -> str:
+        return self.__spec
+
     @property
     def field(self) -> str:
         return self.__field
@@ -181,7 +200,15 @@ class CheckerOP(Enum):
 
 
 class Checker:
-    def __init__(self, checks, optional=False, allow_none=False, op=CheckerOP.ALL, extra=None, **kwargs):
+    def __init__(
+        self,
+        checks: List[str],
+        optional: bool = False,
+        allow_none: bool = False,
+        op: CheckerOP = CheckerOP.ALL,
+        extra: Union[Dict, None] = None,
+        **kwargs,
+    ):
         """
         checks: list of str(Check)
         optional: boolean
@@ -197,16 +224,24 @@ class Checker:
         self._allow_none = allow_none
 
         self._ensure(kwargs)
+
+        check_set = set(checks)
         if extra:
-            print('[DSV][WARNING] keyword: extra is gonna be deprecated')
+            warnings.warn('[DSV] keyword: extra is gonna be deprecated', DeprecationWarning)
         deprecating_checks = {KEY_COEXISTS, ANY_KEY_EXISTS}
-        for deprecating_check in deprecating_checks.intersection(set(checks)):
-            print(f'[DSV][WARNING] Check: {deprecating_check.upper()} is gonna be deprecated')
+        for deprecating_check in deprecating_checks.intersection(check_set):
+            if deprecating_check == KEY_COEXISTS:
+                warnings.warn(f'[DSV] Use COND_EXIST instead of {deprecating_check.upper()} ', DeprecationWarning)
+            elif deprecating_check == ANY_KEY_EXISTS:
+                warnings.warn(
+                    f'[DSV] Use @dsv_feature(any_keys_set...) instead of {deprecating_check.upper()}',
+                    DeprecationWarning,
+                )
 
         self.extra = self._merge_extra_kwargs(extra or {}, kwargs)
 
     @staticmethod
-    def _merge_extra_kwargs(deprecated_extra: dict, check_kwargs: dict) -> dict:
+    def _merge_extra_kwargs(deprecated_extra: Dict, check_kwargs: Dict) -> Dict:
         extra = deprecated_extra.copy()
 
         all_keys = set(_get_check_2_validator_map().keys())
@@ -216,13 +251,12 @@ class Checker:
                 extra[lower_arg_k] = arg_v
         return extra
 
-    def _ensure(self, check_kwargs):
+    def _ensure(self, check_kwargs: Dict):
         def __ensure_upper_case(_kwargs):
             non_upper = list(filter(lambda k: not k.isupper(), _kwargs.keys()))
-            if non_upper:
-                raise TypeError(f'Keyword must be upper-cased: {non_upper}')
+            raise_if(bool(non_upper), TypeError(f'Keyword must be upper-cased: {non_upper}'))
 
-        def __ensure_no_repeated_forbidden(_kwargs):
+        def __ensure_no_repeated_forbidden(_kwargs: Dict):
             blacklist = {'optional', 'allow_none', 'op', 'extra'}
 
             def _check_in_blacklist(acc, key):
@@ -231,12 +265,12 @@ class Checker:
                 return acc
 
             forbidden = list(reduce(_check_in_blacklist, _kwargs.keys(), set()))
-            if forbidden:
-                forbidden.sort()
-                raise TypeError(f'Forbidden keyword arguments: {", ".join(forbidden)}')
+            forbidden.sort()
+            raise_if(bool(forbidden), TypeError(f'Forbidden keyword arguments: {", ".join(forbidden)}'))
 
-        if self._optional and len(self.checks) == 0:
-            raise ValueError('Require at least 1 check when set optional to True')
+        raise_if(
+            self._optional and len(self.checks) == 0, ValueError('Require at least 1 check when set optional=True')
+        )
 
         __ensure_upper_case(check_kwargs)
         __ensure_no_repeated_forbidden(check_kwargs)
@@ -271,7 +305,7 @@ def get_msg_level() -> MsgLv:
     return __message_level
 
 
-def reset_msg_level(vague=False):
+def reset_msg_level(vague: bool = False):
     """
     Setting vague=True, all error messages will be replaced to 'field: XXX not well-formatted'.
     Otherwise, the message is as usual showing the reason.
