@@ -1,8 +1,9 @@
+import copy
 import json
 import re
 import uuid
 from decimal import Decimal
-from typing import List, Tuple, Union
+from typing import Any, Dict, Iterable, List, Tuple, Type, Union
 
 import dateutil.parser
 
@@ -19,6 +20,7 @@ from .defines import (
     DIGIT_STR,
     DUMMY,
     EMAIL,
+    FOREACH,
     INT,
     JSON,
     JSON_BOOL,
@@ -40,16 +42,14 @@ from .defines import (
     get_validator,
 )
 from .features import get_any_keys_set, is_strict
+from .utils import raise_if
 
-
-def _raise_if_condition(condition, message, error_cls=RuntimeError):
-    if condition:
-        raise error_cls(message)
+_ALLOW_UNKNOWN = 'ALLOW_UNKNOWN'
+_SPEC_WISE_CHECKS = [COND_EXIST, KEY_COEXISTS, ANY_KEY_EXISTS]
 
 
 def _extract_fields(spec) -> List[str]:
-    if type(spec) != type:
-        raise RuntimeError(f'{spec} should be just a class')
+    raise_if(type(spec) != type, RuntimeError(f'{spec} should be just a class'))
 
     return list(filter(lambda f: type(f) == str and not (f.startswith('__') and f.endswith('__')), spec.__dict__))
 
@@ -65,43 +65,74 @@ def _extract_value(checks: list, data: dict, field: str):
     return value
 
 
-def __validate_field(data, field, spec) -> Tuple[bool, List[ValidateResult]]:
-    checker = getattr(spec, field)
-
-    checks = checker.checks
-    extra = checker.extra
-
+def _makeup_internals_to_extra(spec: Type, checks: List[str], raw_extra: Dict, allow_optional: bool) -> Dict:
+    extra = copy.deepcopy(raw_extra)
     if extra.get(SpecValidator.name) == SELF:
         extra[SpecValidator.name] = spec
 
+    if COND_EXIST in checks and allow_optional:
+        extra[_ALLOW_UNKNOWN] = True
+    return extra
+
+
+def _pass_optional(allow_optional: bool, checks: List[str], value: Any) -> bool:
+    return value == get_unknown_field_value() and allow_optional and COND_EXIST not in checks
+
+
+def _pass_none(allow_none: bool, value: Any) -> bool:
+    return value is None and allow_none
+
+
+def _pass_unknown(_extra: Dict, value: Any) -> bool:
+    return value == get_unknown_field_value() and _ALLOW_UNKNOWN in _extra
+
+
+def _validate_field(data, field, spec) -> Tuple[bool, List[ValidateResult]]:
+    checker = getattr(spec, field)
+
+    checks = checker.checks
+    allow_optional = checker.allow_optional
+    allow_none = checker.allow_none
+
     value = _extract_value(checks, data, field)
+    extra = _makeup_internals_to_extra(spec, checks, checker.extra, allow_optional)
 
     results = []
 
-    if COND_EXIST in checks and checker.allow_optional:
-        extra['ALLOW_UNKNOWN'] = True
-
-    if value == get_unknown_field_value() and checker.allow_optional and COND_EXIST not in checks:
-        # Pass the checker's validation directly
-        pass
-    elif value is None and checker.allow_none:
-        # Pass the checker's validation directly
-        pass
+    if _pass_optional(allow_optional, checks, value):
+        # Skip all the other checks' validations
+        return True, []
+    elif _pass_none(allow_none, value):
+        # Skip all the other checks' validations
+        return True, []
     else:
-        for check in checks:
-            validator = get_validator(check)
+
+        def _do_validate(_acc_results: List, _spec: Any, _check: str, _value: Any, _data: Dict, _extra: Dict) -> None:
+            validator = get_validator(_check)
             try:
-                ok, error = validator.validate(value, extra, data)
+                ok, error = validator.validate(_value, _extra, _data)
             except AttributeError as ae:
-                if check == LIST_OF:
+                if _check == LIST_OF:
                     # During list_of check, the target should be one kind of spec.
-                    ok, error = False, TypeError(f'{repr(value)} is not a spec of {spec}, detail: {repr(ae)}')
+                    ok, error = False, TypeError(f'{repr(_value)} is not a spec of {_spec}, detail: {repr(ae)}')
                 else:
                     ok, error = False, RuntimeError(f'{repr(ae)}')
+            except NotImplementedError:
+                raise
             except Exception as e:
                 # For any unwell-handled case, go this way for now.
                 ok, error = False, RuntimeError(f'{repr(e)}')
-            results.append((ok, ValidateResult(spec, field, value, check, error)))
+            _acc_results.append((ok, ValidateResult(spec, field, _value, _check, error)))
+
+        spec_wise_checks = set(filter(lambda c: c in _SPEC_WISE_CHECKS, checks))
+        field_wise_checks = set(checks) - spec_wise_checks
+
+        for chk in spec_wise_checks:
+            _do_validate(results, spec, chk, value, data, extra)
+
+        if not _pass_unknown(extra, value):
+            for chk in field_wise_checks:
+                _do_validate(results, spec, chk, value, data, extra)
 
     nok_results = [rs for (ok, rs) in results if not ok]
     if checker.is_op_any and len(nok_results) == len(checks):
@@ -116,7 +147,7 @@ def _validate_spec_features(data, fields, spec) -> Tuple[bool, List[ValidateResu
         unexpected = set(data.keys()) - set(fields)
         if unexpected:
             error = ValueError(f'Unexpected field keys({unexpected}) found in strict mode spec')
-            return False, [ValidateResult(spec, unexpected, data, 'strict', error)]
+            return False, [ValidateResult(spec, str(unexpected), data, 'strict', error)]
 
     any_keys_set = get_any_keys_set(spec)
     if any_keys_set:
@@ -131,7 +162,7 @@ def _validate_spec_features(data, fields, spec) -> Tuple[bool, List[ValidateResu
 
 
 def _validate_spec_fields(data, fields, spec) -> List[Tuple[bool, List[ValidateResult]]]:
-    rs = [__validate_field(data, f, spec) for f in fields]
+    rs = [_validate_field(data, f, spec) for f in fields]
     return rs
 
 
@@ -149,7 +180,7 @@ class IntValidator(BaseValidator):
     @staticmethod
     def validate(value, extra, data) -> Tuple[bool, Union[Exception, str]]:
         ok = type(value) is int
-        info = '' if ok else TypeError(f'{repr(value)} is not a integer')
+        info = '' if ok else TypeError(f'{repr(value)} is not an integer')
         return ok, info
 
 
@@ -246,9 +277,9 @@ class AmountRangeValidator(BaseValidator):
     @staticmethod
     def validate(value, extra, data) -> Tuple[bool, Union[Exception, str]]:
         amount_range_info = extra.get(AmountRangeValidator.name)
-        _raise_if_condition(
+        raise_if(
             type(amount_range_info) != dict or ('min' not in amount_range_info and 'max' not in amount_range_info),
-            f'Invalid extra configuration: {extra}',
+            RuntimeError(f'Invalid extra configuration: {extra}'),
         )
 
         lower_bound = amount_range_info.get('min', float('-inf'))
@@ -265,15 +296,15 @@ class LengthValidator(BaseValidator):
     @staticmethod
     def validate(value, extra, data) -> Tuple[bool, Union[Exception, str]]:
         length_info = extra.get(LengthValidator.name)
-        _raise_if_condition(
+        raise_if(
             type(length_info) != dict or ('min' not in length_info and 'max' not in length_info),
-            f'Invalid extra configuration: {extra}',
+            RuntimeError(f'Invalid extra configuration: {extra}'),
         )
 
         lower_bound, upper_bound = length_info.get('min', 0), length_info.get('max')
-        _raise_if_condition(
+        raise_if(
             lower_bound < 0,
-            'Lower boundary cannot less than 0 for length validator',
+            RuntimeError('Lower boundary cannot less than 0 for length validator'),
         )
 
         ok = lower_bound <= len(value) <= upper_bound if upper_bound else lower_bound <= len(value)
@@ -306,6 +337,9 @@ class ListOfValidator(BaseValidator):
 
     @staticmethod
     def validate(values, extra, data) -> Tuple[bool, Union[Exception, str]]:
+        if type(values) != list:
+            return False, TypeError('Must a be in type: list')
+
         check = extra.get(ListOfValidator.name)
         validator = get_validator(check)
         for value in values:
@@ -327,6 +361,21 @@ class OneOfValidator(BaseValidator):
         return ok, info
 
 
+class ForeachValidator(BaseValidator):
+    name = FOREACH
+
+    @staticmethod
+    def validate(values: Iterable, extra: Dict, data: Dict) -> Tuple[bool, Union[Exception, str]]:
+        check = extra.get(ForeachValidator.name)
+        validator = get_validator(check)
+        for value in values:
+            ok, error = validator.validate(value, extra, data)
+            if not ok:
+                # Early return to save lives.
+                return False, error
+        return True, ''
+
+
 class DecimalPlaceValidator(BaseValidator):
     name = DECIMAL_PLACE
 
@@ -359,16 +408,16 @@ class DateRangeValidator(BaseValidator):
     @staticmethod
     def validate(value, extra, data) -> Tuple[bool, Union[Exception, str]]:
         range_info = extra.get(DateRangeValidator.name)
-        _raise_if_condition(
+        raise_if(
             type(range_info) != dict or ('min' not in range_info and 'max' not in range_info),
-            f'Invalid extra configuration: {extra}',
+            RuntimeError(f'Invalid extra configuration: {extra}'),
         )
 
         min_date_str = range_info.get('min', '1970-01-01')
         max_date_str = range_info.get('max', '2999-12-31')
-        _raise_if_condition(
+        raise_if(
             type(min_date_str) != str or type(max_date_str) != str,
-            f'Invalid extra configuration(must be str): {extra}',
+            RuntimeError(f'Invalid extra configuration(must be str): {extra}'),
         )
 
         min_date = dateutil.parser.parse(min_date_str).date()
@@ -459,7 +508,7 @@ class RegexValidator(BaseValidator):
         elif match_method == 'search':
             match_func = re.search
         else:
-            _raise_if_condition(True, f'unsupported match method: {match_method}')
+            raise RuntimeError(f'unsupported match method: {match_method}')
 
         ok = type(value) == str and match_func and match_func(pattern, value)
         info = '' if ok else ValueError(f'{repr(value)} does not match "{error_regex_param}"')
@@ -471,7 +520,7 @@ class CondExistValidator(BaseValidator):
 
     @staticmethod
     def validate(value, extra, data) -> Tuple[bool, Union[Exception, str]]:
-        allow_unknown = extra.get('ALLOW_UNKNOWN', False)
+        allow_unknown = extra.get(_ALLOW_UNKNOWN, False)
         params = extra.get(CondExistValidator.name, {})
         must_with_keys = params.get('WITH', [])
         must_without_keys = params.get('WITHOUT', [])
