@@ -13,7 +13,9 @@ from data_spec_validator.spec import (
     DATE_RANGE,
     DECIMAL_PLACE,
     DICT,
+    DIGIT_STR,
     EMAIL,
+    FOREACH,
     INT,
     JSON,
     JSON_BOOL,
@@ -30,6 +32,8 @@ from data_spec_validator.spec import (
     UUID,
     Checker,
     CheckerOP,
+    DSVError,
+    ErrorMode,
     dsv_feature,
     not_,
     reset_msg_level,
@@ -164,17 +168,14 @@ class TestSpec(unittest.TestCase):
         assert is_something_error(Exception, validate_data_spec, nok_data, _get_no_extra_self_spec())
 
     def test_list(self):
-        def _get_list_spec():
-            class ListSpec:
-                list_field = Checker([LIST])
-
-            return ListSpec
+        class ListSpec:
+            list_field = Checker([LIST])
 
         ok_data = dict(list_field=[1, 2, 3])
-        assert validate_data_spec(ok_data, _get_list_spec())
+        assert validate_data_spec(ok_data, ListSpec)
 
         nok_data = dict(list_field=dict(a=2, b=4))
-        assert is_something_error(TypeError, validate_data_spec, nok_data, _get_list_spec())
+        assert is_something_error(TypeError, validate_data_spec, nok_data, ListSpec)
 
     def test_dict(self):
         def _get_dict_spec():
@@ -479,25 +480,51 @@ class TestSpec(unittest.TestCase):
         assert is_something_error(TypeError, validate_data_spec, nok_data, _get_list_of_spec_spec())
         assert is_something_error(TypeError, validate_data_spec, nok_data, _get_no_extra_list_of_spec_spec())
 
-        def _get_list_of_int_spec():
-            class ListOfIntSpec:
-                list_of_int_field = Checker([LIST_OF], extra={LIST_OF: INT})
+        class ListOfIntSpec:
+            list_of_int_field = Checker([LIST_OF], extra={LIST_OF: INT})
 
-            return ListOfIntSpec
-
-        def _get_no_extra_list_of_int_spec():
-            class ListOfIntSpec:
-                list_of_int_field = Checker([LIST_OF], LIST_OF=INT)
-
-            return ListOfIntSpec
+        class ListOfIntNoExtraSpec:
+            list_of_int_field = Checker([LIST_OF], LIST_OF=INT)
 
         ok_data = dict(list_of_int_field=[1, 2, 3])
-        assert validate_data_spec(ok_data, _get_list_of_int_spec())
-        assert validate_data_spec(ok_data, _get_no_extra_list_of_int_spec())
+        assert validate_data_spec(ok_data, ListOfIntSpec)
+        assert validate_data_spec(ok_data, ListOfIntNoExtraSpec)
+
+        nok_with_non_list_data = dict(list_of_int_field={1: 1, 2: 2, 3: 3})
+        assert is_something_error(TypeError, validate_data_spec, nok_with_non_list_data, ListOfIntSpec)
 
         nok_data = dict(list_of_int_field=[1, 2, '3'])
-        assert is_something_error(TypeError, validate_data_spec, nok_data, _get_list_of_int_spec())
-        assert is_something_error(TypeError, validate_data_spec, nok_data, _get_no_extra_list_of_int_spec())
+        assert is_something_error(TypeError, validate_data_spec, nok_data, ListOfIntSpec)
+        assert is_something_error(TypeError, validate_data_spec, nok_data, ListOfIntNoExtraSpec)
+
+    def test_foreach(self):
+        class ChildSpec:
+            int_field = Checker([INT])
+            bool_field = Checker([BOOL])
+
+        class ParentSpec:
+            foreach_spec_field = Checker([FOREACH], FOREACH=SPEC, SPEC=ChildSpec)
+
+        ok_data = dict(
+            foreach_spec_field=(
+                dict(int_field=1, bool_field=False),
+                dict(int_field=2, bool_field=True),
+                dict(int_field=3, bool_field=False),
+            )
+        )
+        assert validate_data_spec(ok_data, ParentSpec)
+
+        class ForeachIntSpec:
+            foreach_spec_field = Checker([FOREACH], FOREACH=INT)
+
+        ok_data = dict(foreach_spec_field=(1, 2, 3))
+        assert validate_data_spec(ok_data, ForeachIntSpec)
+        ok_data = dict(foreach_spec_field=[1, 2, 3])
+        assert validate_data_spec(ok_data, ForeachIntSpec)
+        ok_data = dict(foreach_spec_field={1, 2, 3})
+        assert validate_data_spec(ok_data, ForeachIntSpec)
+        ok_data = dict(foreach_spec_field={1: 1, 2: 2, 3: 3})
+        assert validate_data_spec(ok_data, ForeachIntSpec)
 
     def test_one_of(self):
         def _get_one_of_spec():
@@ -995,31 +1022,12 @@ class TestSpec(unittest.TestCase):
 
             return AnyKeyExistsSpec
 
-        def _get_no_extra_any_key_exists_spec():
-            class AnyKeyExistsSpec:
-                test_checker = Checker([ANY_KEY_EXISTS], ANY_KEY_EXISTS={'key1', 'key2', 'key3'})
-
-            return AnyKeyExistsSpec
-
         ok_data = dict(key1=1)
-        assert validate_data_spec(ok_data, _get_any_key_exists_spec())
-        assert validate_data_spec(ok_data, _get_no_extra_any_key_exists_spec())
-
-        ok_data = dict(key2=1, key3=1)
-        assert validate_data_spec(ok_data, _get_any_key_exists_spec())
-        assert validate_data_spec(ok_data, _get_no_extra_any_key_exists_spec())
-
-        nok_data = dict(key=1)
-        assert is_something_error(LookupError, validate_data_spec, nok_data, _get_any_key_exists_spec())
-        assert is_something_error(LookupError, validate_data_spec, nok_data, _get_no_extra_any_key_exists_spec())
+        with self.assertRaises(NotImplementedError) as ctx:
+            validate_data_spec(ok_data, _get_any_key_exists_spec())
+        assert type(ctx.exception) == NotImplementedError
 
     def test_key_coexist(self):
-        def _get_key_coexist_spec():
-            class KeyCoexistsSpec:
-                key1 = Checker([KEY_COEXISTS], extra={KEY_COEXISTS: ['key2']})
-
-            return KeyCoexistsSpec
-
         def _get_no_extra_key_coexist_spec():
             class KeyCoexistsSpec:
                 key1 = Checker([KEY_COEXISTS], KEY_COEXISTS=['key2'])
@@ -1027,16 +1035,9 @@ class TestSpec(unittest.TestCase):
             return KeyCoexistsSpec
 
         ok_data = dict(key1=1, key2=1)
-        assert validate_data_spec(ok_data, _get_key_coexist_spec())
-        assert validate_data_spec(ok_data, _get_no_extra_key_coexist_spec())
-
-        nok_data = dict(key2=1)
-        assert is_something_error(LookupError, validate_data_spec, nok_data, _get_key_coexist_spec())
-        assert is_something_error(LookupError, validate_data_spec, nok_data, _get_no_extra_key_coexist_spec())
-
-        nok_data = dict(key=1)
-        assert is_something_error(LookupError, validate_data_spec, nok_data, _get_key_coexist_spec())
-        assert is_something_error(LookupError, validate_data_spec, nok_data, _get_no_extra_key_coexist_spec())
+        with self.assertRaises(NotImplementedError) as ctx:
+            validate_data_spec(ok_data, _get_no_extra_key_coexist_spec())
+        assert type(ctx.exception) == NotImplementedError
 
     def test_not_checker(self):
         def _get_non_bool_spec():
@@ -1119,6 +1120,15 @@ class TestSpec(unittest.TestCase):
         assert is_something_error(ValueError, validate_data_spec, nok_data_leaf, _RootStrictSpec)
 
     def test_any_keys_set(self):
+        @dsv_feature(any_keys_set={('a', 'b')})
+        class _AnyKeysSetEmptyFieldsSpec:
+            pass
+
+        assert validate_data_spec(dict(a=1, b=1), _AnyKeysSetEmptyFieldsSpec)
+        assert validate_data_spec(dict(a=1), _AnyKeysSetEmptyFieldsSpec)
+        assert validate_data_spec(dict(b=1), _AnyKeysSetEmptyFieldsSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(c=1), _AnyKeysSetEmptyFieldsSpec)
+
         @dsv_feature(any_keys_set={('a', 'b')})
         class _AnyKeysSetSpec:
             a = Checker([INT], optional=True)
@@ -1127,7 +1137,6 @@ class TestSpec(unittest.TestCase):
         assert validate_data_spec(dict(a=1, b=1), _AnyKeysSetSpec)
         assert validate_data_spec(dict(a=1), _AnyKeysSetSpec)
         assert validate_data_spec(dict(b=1), _AnyKeysSetSpec)
-
         assert is_something_error(LookupError, validate_data_spec, dict(c=1), _AnyKeysSetSpec)
 
         @dsv_feature(any_keys_set={('a', 'b'), ('c', 'd')})
@@ -1153,6 +1162,65 @@ class TestSpec(unittest.TestCase):
         assert is_something_error(LookupError, validate_data_spec, dict(d=1), _AnyKeysSetSpec)
         assert is_something_error(LookupError, validate_data_spec, dict(e=1), _AnyKeysSetSpec)
 
+    def test_err_mode(self):
+        @dsv_feature(err_mode=ErrorMode.ALL)
+        class _ErrModeAllSpec:
+            a = Checker([INT])
+            b = Checker([DIGIT_STR])
+            c = Checker([LENGTH, STR, AMOUNT], LENGTH=dict(min=3, max=5))
+
+        nok_data = dict(
+            a=True,
+            b='abc',
+            c='22',
+        )
+
+        with self.assertRaises(DSVError) as context:
+            validate_data_spec(nok_data, _ErrModeAllSpec)
+        assert len(context.exception.args) == 3
+
+        def _get_nested_err_mode_spec(mode):
+            @dsv_feature()
+            class LeafSpec:
+                int_f = Checker([INT])
+                str_f = Checker([STR])
+                bool_f = Checker([BOOL])
+
+            class MidLeafSpec:
+                int_f = Checker([INT])
+                str_f = Checker([STR])
+                leaf_f = Checker([SPEC], SPEC=LeafSpec)
+
+            @dsv_feature(err_mode=mode)
+            class RootSpec:
+                int_f = Checker([INT])
+                ml_f = Checker([SPEC], SPEC=MidLeafSpec)
+                bool_f = Checker([BOOL])
+
+            return RootSpec
+
+        nok_data2 = dict(
+            int_f='a',
+            ml_f=dict(
+                int_f=3.3,
+                str_f='ok',
+                leaf_f=dict(
+                    int_f=1,
+                    str_f=True,
+                    bool_f='non-bool',
+                ),
+            ),
+            bool_f='22',
+        )
+
+        with self.assertRaises(DSVError) as context:
+            validate_data_spec(nok_data2, _get_nested_err_mode_spec(ErrorMode.ALL))
+        assert len(context.exception.args) == 5
+
+        with self.assertRaises(TypeError) as context:
+            validate_data_spec(nok_data2, _get_nested_err_mode_spec(ErrorMode.MSE))
+        assert len(context.exception.args) == 1
+
     def test_conditional_existence(self):
         """
         The existence cases of a, b, c. 2 * 2 * 2 = 8 cases.
@@ -1291,6 +1359,135 @@ class TestSpec(unittest.TestCase):
         assert is_something_error(LookupError, validate_data_spec, dict(c=1), _CondExistABCSpec)
         assert is_something_error(LookupError, validate_data_spec, dict(d=1), _CondExistABCSpec)
 
+    def test_optional_conditional_existence_other_check_fail(self):
+        """
+        The existence cases of a, b, c. 2 * 2 * 2 = 8 cases.
+        dict(a=1, b=1, c=1)
+        dict(a=1, b=1)
+        dict(a=1, c=1)
+        dict(b=1, c=1)
+        dict(a=1)
+        dict(b=1)
+        dict(c=1)
+        dict(d=1)
+        """
+        # ==========================
+        class _CondExistOtherFailAOBOCOSpec:
+            a = Checker([STR, COND_EXIST], optional=True, COND_EXIST=dict(WITHOUT=['c']))
+            b = Checker([STR, COND_EXIST], optional=True, COND_EXIST=dict(WITH=['a'], WITHOUT=['c']))
+            c = Checker([STR, COND_EXIST], optional=True, COND_EXIST=dict(WITHOUT=['a']))
+
+        assert validate_data_spec(dict(d=1), _CondExistOtherFailAOBOCOSpec)
+        # Spec Wise
+        assert is_something_error(KeyError, validate_data_spec, dict(a=1, b=1, c=1), _CondExistOtherFailAOBOCOSpec)
+        assert is_something_error(KeyError, validate_data_spec, dict(a=1, c=1), _CondExistOtherFailAOBOCOSpec)
+        assert is_something_error(KeyError, validate_data_spec, dict(b=1, c=1), _CondExistOtherFailAOBOCOSpec)
+        assert is_something_error(KeyError, validate_data_spec, dict(b=1), _CondExistOtherFailAOBOCOSpec)
+        # Field Wise
+        assert is_something_error(TypeError, validate_data_spec, dict(a=1, b=1), _CondExistOtherFailAOBOCOSpec)
+        assert is_something_error(TypeError, validate_data_spec, dict(a=1), _CondExistOtherFailAOBOCOSpec)
+        assert is_something_error(TypeError, validate_data_spec, dict(c=1), _CondExistOtherFailAOBOCOSpec)
+
+        # ==========================
+        class _CondExistOtherFailABOCOSpec:
+            a = Checker([STR, COND_EXIST], COND_EXIST=dict(WITHOUT=['c']))
+            b = Checker([STR, COND_EXIST], optional=True, COND_EXIST=dict(WITH=['a'], WITHOUT=['c']))
+            c = Checker([STR, COND_EXIST], optional=True, COND_EXIST=dict(WITHOUT=['a']))
+
+        # SPec Wise
+        assert is_something_error(KeyError, validate_data_spec, dict(a=1, b=1, c=1), _CondExistOtherFailABOCOSpec)
+        assert is_something_error(KeyError, validate_data_spec, dict(a=1, c=1), _CondExistOtherFailABOCOSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(b=1, c=1), _CondExistOtherFailABOCOSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(b=1), _CondExistOtherFailABOCOSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(c=1), _CondExistOtherFailABOCOSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(d=1), _CondExistOtherFailABOCOSpec)
+        # Field Wise
+        assert is_something_error(TypeError, validate_data_spec, dict(a=1, b=1), _CondExistOtherFailABOCOSpec)
+        assert is_something_error(TypeError, validate_data_spec, dict(a=1), _CondExistOtherFailABOCOSpec)
+
+        # ==========================
+        class _CondExistOtherFailAOBCOSpec:
+            a = Checker([STR, COND_EXIST], optional=True, COND_EXIST=dict(WITHOUT=['c']))
+            b = Checker([STR, COND_EXIST], COND_EXIST=dict(WITH=['a'], WITHOUT=['c']))
+            c = Checker([STR, COND_EXIST], optional=True, COND_EXIST=dict(WITHOUT=['a']))
+
+        # SPec Wise
+        assert is_something_error(KeyError, validate_data_spec, dict(a=1, b=1, c=1), _CondExistOtherFailAOBCOSpec)
+        assert is_something_error(KeyError, validate_data_spec, dict(a=1, c=1), _CondExistOtherFailAOBCOSpec)
+        assert is_something_error(KeyError, validate_data_spec, dict(b=1, c=1), _CondExistOtherFailAOBCOSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(a=1), _CondExistOtherFailAOBCOSpec)
+        assert is_something_error(KeyError, validate_data_spec, dict(b=1), _CondExistOtherFailAOBCOSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(c=1), _CondExistOtherFailAOBCOSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(d=1), _CondExistOtherFailAOBCOSpec)
+        # Field Wise
+        assert is_something_error(TypeError, validate_data_spec, dict(a=1, b=1), _CondExistOtherFailAOBCOSpec)
+
+        # ==========================
+        class _CondExistOtherFailAOBOCSpec:
+            a = Checker([STR, COND_EXIST], optional=True, COND_EXIST=dict(WITHOUT=['c']))
+            b = Checker([STR, COND_EXIST], optional=True, COND_EXIST=dict(WITH=['a'], WITHOUT=['c']))
+            c = Checker([STR, COND_EXIST], COND_EXIST=dict(WITHOUT=['a']))
+
+        # Spec Wise
+        assert is_something_error(KeyError, validate_data_spec, dict(a=1, b=1, c=1), _CondExistOtherFailAOBOCSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(a=1, b=1), _CondExistOtherFailAOBOCSpec)
+        assert is_something_error(KeyError, validate_data_spec, dict(a=1, c=1), _CondExistOtherFailAOBOCSpec)
+        assert is_something_error(KeyError, validate_data_spec, dict(b=1, c=1), _CondExistOtherFailAOBOCSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(a=1), _CondExistOtherFailAOBOCSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(b=1), _CondExistOtherFailAOBOCSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(d=1), _CondExistOtherFailAOBOCSpec)
+        # Field Wise
+        assert is_something_error(TypeError, validate_data_spec, dict(c=1), _CondExistOtherFailAOBOCSpec)
+        # ==========================
+
+        class _CondExistOtherFailABCOSpec:
+            a = Checker([STR, COND_EXIST], COND_EXIST=dict(WITHOUT=['c']))
+            b = Checker([STR, COND_EXIST], COND_EXIST=dict(WITH=['a'], WITHOUT=['c']))
+            c = Checker([STR, COND_EXIST], optional=True, COND_EXIST=dict(WITHOUT=['a']))
+
+        # Spec Wise
+        assert is_something_error(KeyError, validate_data_spec, dict(a=1, b=1, c=1), _CondExistOtherFailABCOSpec)
+        assert is_something_error(KeyError, validate_data_spec, dict(a=1, c=1), _CondExistOtherFailABCOSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(b=1, c=1), _CondExistOtherFailABCOSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(a=1), _CondExistOtherFailABCOSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(b=1), _CondExistOtherFailABCOSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(c=1), _CondExistOtherFailABCOSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(d=1), _CondExistOtherFailABCOSpec)
+        # Field Wise
+        assert is_something_error(TypeError, validate_data_spec, dict(a=1, b=1), _CondExistOtherFailABCOSpec)
+
+        # ==========================
+        class _CondExistOtherFailABOCSpec:
+            a = Checker([STR, COND_EXIST], COND_EXIST=dict(WITHOUT=['c']))
+            b = Checker([STR, COND_EXIST], optional=True, COND_EXIST=dict(WITH=['a'], WITHOUT=['c']))
+            c = Checker([STR, COND_EXIST], COND_EXIST=dict(WITHOUT=['a']))
+
+        # Spec Wise
+        assert is_something_error(KeyError, validate_data_spec, dict(a=1, b=1, c=1), _CondExistOtherFailABOCSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(a=1, b=1), _CondExistOtherFailABOCSpec)
+        assert is_something_error(KeyError, validate_data_spec, dict(a=1, c=1), _CondExistOtherFailABOCSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(b=1, c=1), _CondExistOtherFailABOCSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(a=1), _CondExistOtherFailABOCSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(b=1), _CondExistOtherFailABOCSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(c=1), _CondExistOtherFailABOCSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(d=1), _CondExistOtherFailABOCSpec)
+        # ==========================
+
+        class _CondExistOtherFailAOBCSpec:
+            a = Checker([STR, COND_EXIST], optional=True, COND_EXIST=dict(WITHOUT=['c']))
+            b = Checker([STR, COND_EXIST], COND_EXIST=dict(WITH=['a'], WITHOUT=['c']))
+            c = Checker([STR, COND_EXIST], COND_EXIST=dict(WITHOUT=['a']))
+
+        assert is_something_error(KeyError, validate_data_spec, dict(a=1, b=1, c=1), _CondExistOtherFailAOBCSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(a=1, b=1), _CondExistOtherFailAOBCSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(a=1, c=1), _CondExistOtherFailAOBCSpec)
+        assert is_something_error(KeyError, validate_data_spec, dict(b=1, c=1), _CondExistOtherFailAOBCSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(a=1), _CondExistOtherFailAOBCSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(b=1), _CondExistOtherFailAOBCSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(c=1), _CondExistOtherFailAOBCSpec)
+        assert is_something_error(LookupError, validate_data_spec, dict(d=1), _CondExistOtherFailAOBCSpec)
+        # ==========================
+
 
 class TestCustomSpec(unittest.TestCase):
     def test_incorrect_validator_class(self):
